//////////////////////////////////////////////////////////////////////
// Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)
// -- joat
// -- modified by Darkain and others
//////////////////////////////////////////////////////////////////////


#include <stdlib.h>
#include <stdio.h>
 
#include <NDS/NDS.h>
 
#include <NDS/ARM7/BIOS.h>
#include <NDS/ARM7/touch.h>
#include <NDS/ARM7/clock.h>

#include "../../ipc2.h"

//////////////////////////////////////////////////////////////////////


#define TOUCH_CAL_X1 (*(vs16*)0x027FFCD8)
#define TOUCH_CAL_Y1 (*(vs16*)0x027FFCDA)
#define TOUCH_CAL_X2 (*(vs16*)0x027FFCDE)
#define TOUCH_CAL_Y2 (*(vs16*)0x027FFCE0)
#define SCREEN_WIDTH    256
#define SCREEN_HEIGHT   192
s32 TOUCH_WIDTH  = TOUCH_CAL_X2 - TOUCH_CAL_X1;
s32 TOUCH_HEIGHT = TOUCH_CAL_Y2 - TOUCH_CAL_Y1;
s32 TOUCH_OFFSET_X = ( ((SCREEN_WIDTH -60) * TOUCH_CAL_X1) / TOUCH_WIDTH  ) - 28;
s32 TOUCH_OFFSET_Y = ( ((SCREEN_HEIGHT-60) * TOUCH_CAL_Y1) / TOUCH_HEIGHT ) - 28;

//////////////////////////////////////////////////////////////////////

#define IPCSOUND_IRQ_TIMER (IRQ_TIMER0)
#define IPCSOUND_TIMER_CR (TIMER0_CR)
#define IPCSOUND_TIMER_DATA (TIMER0_DATA)


//////////////////////////////////////////////////////////////////////

void startSound(int sampleRate, const void* data, uint32 bytes, u8 channel=0, u8 vol=0x7F,  u8 pan=63, u8 format=0) {
  SCHANNEL_CR(channel)     = 0;
  SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
  SCHANNEL_SOURCE(channel) = (uint32)data;
  SCHANNEL_LENGTH(channel) = bytes;
  SCHANNEL_REPEAT_POINT(channel) = 0;
  SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}

void setparamSound(int sampleRate, u8 channel=0, u8 vol=0x7F,  u8 pan=63, u8 format=0) {
  SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
  SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}


//////////////////////////////////////////////////////////////////////

void InterruptHandler(void) {
  static int heartbeat = 0;
 
  if (IF & IPCSOUND_IRQ_TIMER) {
    
    //sound code  :)
    TransferSound2 *snd = IPC2->soundData;
    if (snd!=0) {
      if(snd->data.data!=NULL){
        startSound(snd->data.rate, snd->data.data, snd->data.len, snd->chan, snd->data.vol, snd->data.pan, snd->data.format);
        }else{
        setparamSound(snd->data.rate, snd->chan, snd->data.vol, snd->data.pan, snd->data.format);
      }
      IPC2->soundData = 0;
    }
    
    u16 SoundEnableFlags=0;
    
    for(u32 i=0; i<16; i++) {
      if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) != 0 ){
        SoundEnableFlags|=1<<i;
      }
    }
    
    IPC2->SoundEnableFlags=SoundEnableFlags;
    
    IF = IPCSOUND_IRQ_TIMER;
    return;
  }
  
  if (IF & IRQ_VBLANK) {
    uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0, batt=0, aux=0;
    int t1=0, t2=0;
    uint32 temp=0;
    uint8 ct[sizeof(IPC2->curtime)];
    

    
    // Update the heartbeat
    heartbeat++;
 
    // Read the X/Y buttons and the /PENIRQ line
    but = XKEYS;
    if (!(but & 0x40)) {
      // Read the touch screen
      x = touchRead(TSC_MEASURE_X);
      y = touchRead(TSC_MEASURE_Y);
      xpx = ( ((SCREEN_WIDTH -60) * x) / TOUCH_WIDTH  ) - TOUCH_OFFSET_X;
      ypx = ( ((SCREEN_HEIGHT-60) * y) / TOUCH_HEIGHT ) - TOUCH_OFFSET_Y;
      z1 = touchRead(TSC_MEASURE_Z1);
      z2 = touchRead(TSC_MEASURE_Z2);
    }

    batt = touchRead(TSC_MEASURE_BATTERY);
    aux  = touchRead(TSC_MEASURE_AUX);
    
    // Read the time
    if(IPC2->curtimeFlag!=0){
      rtcGetTime((uint8 *)ct);
      BCDToInteger((uint8 *)&(ct[1]), 7);
      
      for(u32 i=0; i<sizeof(ct); i++) {
        IPC2->curtime[i] = ct[i];
      }
      
      IPC2->curtimeFlag=0;
    }
 
    // Read the temperature
    temp = touchReadTemperature(&t1, &t2);
 
    // Update the IPC struct
    IPC2->heartbeat = heartbeat;
    IPC2->buttons   = but;
    IPC2->touchX    = x;
    IPC2->touchY    = y;
    IPC2->touchXpx  = xpx;
    IPC2->touchYpx  = ypx;
    IPC2->touchZ1   = z1;
    IPC2->touchZ2   = z2;
    IPC2->battery   = batt;
    IPC2->aux       = aux;

    IPC2->temperature = temp;
    IPC2->tdiode1 = t1;
    IPC2->tdiode2 = t2;
    
    IF = IRQ_VBLANK;
    return;
  }
 
  // Acknowledge interrupts
  IF = IF;
}
 

//////////////////////////////////////////////////////////////////////

static inline void ipcsoundSetTimer(void)
{
  IPCSOUND_TIMER_DATA=0xffff-512;
  IPCSOUND_TIMER_CR=TIMER_ENABLE | TIMER_IRQ_REQ | TIMER_DIV_1;
}


int main(int argc, char ** argv) {
  // Reset the clock if needed
  rtcReset();

  //enable sound
  powerON(POWER_SOUND);
  SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
  IPC2->soundData = 0;
  
  IPC2->curtimeFlag=0;
 
  // Set up the interrupt handler
  IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  IE = IRQ_VBLANK | IPCSOUND_IRQ_TIMER;
  IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  IME = 1;
  
  ipcsoundSetTimer();
  
  // Keep the ARM7 out of main RAM
  while (1){
    swiWaitForVBlank();
  }
  return 0;
}

 
//////////////////////////////////////////////////////////////////////

