
#include <stdio.h>
#include <stdlib.h>

#include <NDS/NDS.h>

#include <NDS/ARM9/console.h> //basic print funcionality
#include <NDS/ARM9/CP15.h>

#include "../../ipc2.h"
#include "waveprint.h"

#define SOUND_FORMAT_16BIT (0)
#define SOUND_FORMAT_8BIT (1)

#define waveprint_sch (15)
#define waveprint_rate (22050)
#define waveprint_vol (127)
#define waveprint_pan (63)

static s8 *wpr_pcmdata;
static u32 wpr_pcmcount;
static s8 wpr_curpcm;

void wpr_Init(void)
{
  wpr_pcmdata=(s8*)malloc(12+(256*64)); // 256charbuf
  wpr_pcmcount=0;
  wpr_curpcm=127;
}

void wpr_Free(void)
{
  free(wpr_pcmdata);
  wpr_pcmdata=0;
  wpr_pcmcount=0;
  wpr_curpcm=0;
}

void wpr_DrainPCM(void)
{
  // wpr_pcmdataĐBTCYwpr_pcmcount(byteP)
  // ƎƂ́AŌwpr_pcmcount0̂YȂłB
  
  DC_FlushAll();
  while((IPC2->SoundEnableFlags&(1<<waveprint_sch))!=0){
    DC_FlushAll();
  }
  
  TransferSound2 *snd;
  snd=(pTransferSound2)malloc(sizeof(TransferSound2));
  
  DC_FlushAll();
  while(IPC2->soundData!=0){
    DC_FlushAll();
  }
  
  snd->chan=waveprint_sch;
  snd->data.rate=waveprint_rate;
  snd->data.data=wpr_pcmdata;
  snd->data.len=wpr_pcmcount/4;
  snd->data.vol=waveprint_vol;
  snd->data.pan=waveprint_pan;
  snd->data.format=SOUND_FORMAT_8BIT;
  CP15_DrainWriteBuffer();
  
  IPC2->soundData=snd;
  
  DC_FlushAll();
  while(IPC2->soundData!=0){
    DC_FlushAll();
  }
  
  free(snd);
  
  DC_FlushAll();
  while((IPC2->SoundEnableFlags&(1<<waveprint_sch))!=0){
    DC_FlushAll();
  }
  
  wpr_pcmcount=0;
}

void wpr_AddPulse(u32 bitwidth)
{
  s8 *pcmdata=wpr_pcmdata;
  u32 pcmcount=wpr_pcmcount;
  s8 curpcm=wpr_curpcm;
  
  for(u32 cnt=0;cnt<bitwidth;cnt++){
    pcmdata[pcmcount]=curpcm; pcmcount++;
  }
  
  wpr_pcmcount=pcmcount;
  wpr_curpcm=-wpr_curpcm;
}

static inline void wpr_AddPulse4(void)
{
  s8 *pcmdata=&wpr_pcmdata[wpr_pcmcount];
  s8 curpcm=wpr_curpcm;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  wpr_pcmcount+=4;
  wpr_curpcm=-wpr_curpcm;
}

static inline void wpr_AddPulse8(void)
{
  s8 *pcmdata=&wpr_pcmdata[wpr_pcmcount];
  s8 curpcm=wpr_curpcm;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  wpr_pcmcount+=8;
  wpr_curpcm=-wpr_curpcm;
}

static inline void wpr_AddPulse12(void)
{
  s8 *pcmdata=&wpr_pcmdata[wpr_pcmcount];
  s8 curpcm=wpr_curpcm;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  *pcmdata=curpcm; pcmdata++;
  
  wpr_pcmcount+=12;
  wpr_curpcm=-wpr_curpcm;
}

void wpr_WriteByte(u8 data)
{
  u32 bitcount;
  
  wpr_AddPulse12();
  
  for(bitcount=0;bitcount<8;bitcount++){
    if((data&(1<<bitcount))==0){
      wpr_AddPulse8();
      }else{
      wpr_AddPulse4();
    }
  }
}

void wpr_WriteString(const char *data)
{
  while(*data!=0){
    wpr_WriteByte((u8)*data);
    data++;
  }
}

void wpr_WriteBinary(void *data,u32 bytes)
{
  u32 cnt;
  
  for(cnt=0;cnt<bytes;cnt++){
    wpr_WriteByte(((u8*)data)[cnt]);
  }
}

